package main

import (
	"fmt"
	"log"
	"main/mvc/controllers"
	"main/mvc/controllers/controllers_pages"
	"main/mvc/models"
	"main/tools"
	"net/http"
	"os"
	"time"
)

func main() {
	var err error
	var app *models.App
	var logFile *os.File
	defer logFile.Close()

	// Инициализация приложения
	if app, err = models.InitApp(); err != nil {
		log.Fatal(err)
	}
	// Запуск тикеров
	{
		// Переоткрытие файла с логами периодически чтобы он не стал слишком большим
		tools.Ticker(func() {
			newLogFile, err := tools.SetupLogging(app.Cfg.LogFilePath)
			if err != nil {
				log.Fatal(err)
			}
			logFile.Close()
			logFile = newLogFile
			log.Println("Logging system restarted")
		}, time.Second*app.Cfg.LogSystemRestartInterval)

		// Обновление последнего прослушанного трека ластфм
		tools.Ticker(func() {
			var lastTrack string
			if lastTrack, err = models.LastFMGetLastTrack(app.Cfg.LastFMUsername, app.Cfg.LastFMToken); err != nil {
				log.Println(err)
				return
			}
			app.LastfmLastTrack = lastTrack
		}, time.Second*app.Cfg.LastFMUpdateInterval)

		// Сообщение в лог о количестве элементов в кэше
		tools.Ticker(func() {
			log.Println("Pages in cache:")
			if len(app.PagesCache.Data) != 0 {
				for key := range app.PagesCache.Data {
					log.Println(key)
				}
			} else {
				log.Println("No pages in the cache")
			}
		}, time.Second*app.Cfg.CacheLogInterval)
	}

	// Настройка маршрутов
	router := setupRoutes(app)

	// Запуск сервера
	if ok, err := tools.IsIPInUse(app.Cfg.ServerIP); err != nil {
		log.Fatal(err)
	} else if ok {
		runServer(app.Cfg.ServerIP, app.Cfg.ServerPort, router)
	} else {
		runServer(app.Cfg.LocalIP, app.Cfg.LocalPort, router)
	}
}

// Настраивает маршруты
func setupRoutes(app *models.App) *http.ServeMux {
	router := http.NewServeMux()

	// Цепочка обработчиков, которые сработают до отдачи страницы юзеру
	m := controllers.MiddlewaresChain

	// Обработка статических файлов
	router.Handle(app.Cfg.AssetsDir, m(controllers.StaticHandler()))

	// Главные странички
	{
		// Обработка главной страницы
		router.Handle("/", m(controllers_pages.MainPageHandler(app)))
	}

	// Странички со списками постов
	postsPagesCount := (len(app.PostsMap) + app.Cfg.PostsMaxCountOnPage - 1) / app.Cfg.PostsMaxCountOnPage
	for i := range postsPagesCount {
		router.Handle(
			fmt.Sprintf("/posts/%d", i),
			m(controllers_pages.PostsPageHandler(app)))
	}

	// Обработка страничек постов
	for key := range app.PostsMap {
		postLink := string(key)
		router.Handle(postLink, m(controllers_pages.PostPageHandler(app)))
	}

	// Api
	{
		router.Handle("/api/lastfm", m(controllers.LastFMHandler(app)))
	}

	return router
}

// Запускает сервер на указанном IP, PORT
func runServer(ip, port string, router http.Handler) {
	addr := ip + port
	log.Println("Run on", addr)
	log.Fatal(http.ListenAndServe(addr, router))
}
